#include <strstream>

#include "qospf-agent.h"
#include "agent.h"
#include "hdr-qospf.h"


// boiler plate code for a class exported to tcl namespace
// see ns-manual -> OTcl Linkage -> Class TclClass

static class rtProtoQOSPFclass: public TclClass {
public:
    rtProtoQOSPFclass(): TclClass("Agent/QOSPF") {}
    TclObject* create(int, const char*const*) {
	// Agent/rtProto/QOSPF maps to rtProtoQOSPF
	return (new rtProtoQOSPF);
    }
} class_rtProtoQOSPF;

int rtProtoQOSPF::command(int argc, const char* const* argv) {
    //cout << "in rtProtoQOSPF::command" << endl;
    //cout << "argc = " << argc << " argv = ";
    //for (int i = 0; i < argc; i++) cout << argv[i] << " ";
    //cout << endl;

    if (strcmp(argv[1], "initialize") == 0) {
	initialize ();
	return TCL_OK;
    }
    if (strcmp(argv[1], "setNodeNumber" ) == 0 ) {
	if ( argc == 3 ) {
	    int number_of_nodes = atoi(argv[2]);
	    QOSPF::LsMessageCenter::instance().setNodeNumber(number_of_nodes);
	}
	return TCL_OK;
    }
    if (strcmp(argv[1], "lookupBw" ) == 0 ) {
	if ( argc == 3 ) {
	    int neighbor = atoi(argv[2]);
	    strstream s;
	    s << routing_.getBw(neighbor) << '\0';
	    Tcl::instance().result(s.str());
	    return TCL_OK;
	}
    }
    if (strcmp(argv[1], "isBufferEmpty" ) == 0 ) {
	if ( argc == 2 ) {
	    strstream s;
	    s << ( routing_.isBufferEmpty() ? 1 : 0 ) << '\0';
	    Tcl::instance().result(s.str());
	    return TCL_OK;
	}
    }
    if (strcmp(argv[1], "setDelay" ) == 0 ) {
	if ( argc == 4) {
	    int nbrId = atoi(argv[2]);
	    double delay = strtod ( argv[3], NULL);
	    setDelay(nbrId, delay);
	    return TCL_OK;
	}
    }
    if (strcmp(argv[1], "intfChanged") == 0) {
	intfChanged();
	return TCL_OK;
    } 
    if (strcmp(argv[1], "dumpmap") == 0) {
	routing_.dumpmap();
	return TCL_OK;
    } 
    if (strcmp(argv[1], "computeRoutes") == 0) {
	computeQOSRoutes();
	return TCL_OK;
    }
    if (strcmp (argv[1], "send-buffered-messages") == 0) {
	sendBufferedMessages();
	return TCL_OK;
    }  
    if (strcmp(argv[1], "sendUpdates") == 0) {
	sendUpdates ();
	return TCL_OK;
    }
    if (strcmp(argv[1], "get-pre-explicit-route") == 0) {
	int n = atoi(argv[2]);
	double b = strtod(argv[3], NULL);
	list<int> eroute = routing_.get_pre_explicit_route(n, b);
	//printf ("in eroute %d %f %d\n", n, b, eroute.size());
	strstream s;
	for (list<int>::iterator i = eroute.begin(); 
	     i != eroute.end();
	     i ++) {
	    s << int(*i) << " ";
	}
	s << '\0';
	Tcl::instance().result(s.str());
	return TCL_OK;
    }
    if (strcmp(argv[1], "get-od-explicit-route") == 0) {
	int n = atoi(argv[2]);
	double b = strtod(argv[3], NULL);
	list<int> eroute = routing_.get_od_explicit_route(n, b);
	//printf ("in eroute %d %f %d\n", n, b, eroute.size());
	strstream s;
	for (list<int>::iterator i = eroute.begin(); 
	     i != eroute.end();
	     i ++) {
	    s << int(*i) << " ";
	}
	s << '\0';
	Tcl::instance().result(s.str());
	return TCL_OK;
    }
  
  

    return Agent::command(argc, argv);
}

void rtProtoQOSPF::initialize() {
    Tcl& tcl = Tcl::instance();

    // 1. set nodeId_
    tcl.evalf("%s get-node-id", name());
    char * resultString = tcl.result();
    nodeId_ = atoi(resultString);
  
    // 2. fill peerAddrMap_, peerIdList_
    tcl.evalf("%s get-peers", name());
    strstream tclresult;
    tclresult << tcl.result();
    int nodeid;
    ns_addr_t peer;

    while(tclresult) {
	tclresult >> nodeid;
	tclresult >> peer.addr_;
	tclresult >> peer.port_;
	if (!tclresult) break;
// 	cout << "nodeid = " << nodeid 
// 	     << " addr = " << peer.addr_ 
// 	     << " port = " << peer.port_ << endl;
	peerAddrMap_.insert(nodeid, peer);
	peerIdList_.push_back(nodeid);
    }

    // 3. fill linkStateList_
    tcl.evalf("%s get-links-status", name());
    strstream tclresult1;
    tclresult1 << tcl.result();

    int status, cost;
    QOSPF::ls_status_t stat;
    double bw;

    while(tclresult1) {
	tclresult1 >> nodeid;
	tclresult1 >> status;
	tclresult1 >> cost;
	tclresult1 >> bw;
	if (!tclresult1) break;
// 	cout << "nodeid = " << nodeid
// 	     << " status = " << status
// 	     << " cost = " << cost
// 	     << " bw =" << bw << endl;
	stat = QOSPF::ls_status_t(status);
	linkStateList_.push_back(QOSPF::LsLinkState(nodeid,
						    stat,
						    cost,
						    bw));

    }

    // 4. get delay estimates
    tcl.evalf ("%s get-delay-estimates", name());

    // 5. initialize routing
    routing_.init(this);

    // 6. compute qos routes
    // FIXME do we need to do this now eg. when on-demand computation
    routing_.computeQOSRoutes();

    // 7. set LS_ready_ FIXME dont know why its here
    tcl.evalf("%s set LS_ready", name());
    char* token = strtok(tcl.result(), " \t\n");
    if (token == NULL) 
	LS_ready_ = 0;
    else
	LS_ready_ = atoi(token); // buggy
  

}
    
    
void rtProtoQOSPF::intfChanged()
{
    Tcl& tcl = Tcl::instance();

    tcl.evalf("%s get-links-status", name());
    strstream tclresult1;
    tclresult1 << tcl.result();
  
    int nodeid, status, cost;
    QOSPF::ls_status_t stat;
    double bw;
  
    linkStateList_.erase(linkStateList_.begin(), linkStateList_.end());

    while(tclresult1) {
	tclresult1 >> nodeid;
	tclresult1 >> status;
	tclresult1 >> cost;
	tclresult1 >> bw;
	if (!tclresult1) break;
// 	cout << "nodeid = " << nodeid
// 	     << " status = " << status
// 	     << " cost = " << cost
// 	     << " bw =" << bw << endl;
	stat = QOSPF::ls_status_t(status);
	linkStateList_.push_back(QOSPF::LsLinkState(nodeid,
						    stat,
						    cost,
						    bw));

    }
  
    tcl.evalf("%s getAdvertMinBwChangePercent", name());
    double minbwper = strtod(tcl.result(), NULL);
    routing_.minBwChangePercent = minbwper;
    routing_.linkStateChanged();

}


void rtProtoQOSPF::recv(Packet* p, Handler*)
{   
    hdr_QOSPF* rh = hdr_QOSPF::access(p);
    hdr_ip* ih = hdr_ip::access(p);
    // FIXME look why LS Ready is needed
    if (LS_ready_ || (rh->metricsVar() == QOSPF::LS_BIG_NUMBER))
	receiveMessage(findPeerNodeId(ih->src()), rh->msgId());
    else
	Tcl::instance().evalf("%s recv-update %d %d", name(),
			      ih->saddr(), rh->metricsVar());
    Packet::free(p);
}

int rtProtoQOSPF::findPeerNodeId (ns_addr_t agentAddr) 
{
    // because the agentAddr is the value, not the key of the map
    for (PeerAddrMap::iterator itr = peerAddrMap_.begin();
	 itr != peerAddrMap_.end(); itr++) {
	if ((*itr).second == agentAddr) {
	    return (*itr).first;
	}
    }
    return QOSPF::LS_INVALID_NODE_ID;
}

void rtProtoQOSPF::receiveMessage(int sender, u_int32_t msgId) 
{ 
  
    routing_.receiveMessage(sender, msgId);
  
}

bool rtProtoQOSPF::sendMessage(int destId, u_int32_t messageId, int size) 
{

    //cout << "in node " << nodeId_ << endl;

    ns_addr_t* agentAddrPtr = peerAddrMap_.findPtr(destId);
    if (agentAddrPtr == NULL) 
	return false;
    dst_ = *agentAddrPtr;
    size_ = size;
  
    Packet* p = Agent::allocpkt();
    hdr_QOSPF *rh = hdr_QOSPF::access(p);
    rh->msgId() = messageId;
    rh->metricsVar() = QOSPF::LS_BIG_NUMBER;
    target_->recv(p);           
    // sendpkt( *agentAddrPtr , messageId, size);
    return true;
}
