# The topology :
#
#       100Mb 1ms	          	      100Mb 1ms
#   s(0)-----------.	  8Mb    8Mb     .-------------d(0)
# 		   |    ______c2_____    |	
#      100Mb 1ms   |   /  4ms    4ms \   |    100Mb 1ms
#   s(1)--------.  |  /		      \  |  .----------d(1)
# 	         \ | /                 \ | /
#      100Mb 1ms  \|/ 10Mb 5ms  10Mb 5ms\|/   100Mb 1ms
#   s(2)----------e1----------c1---------e2------------d(2)
#                 /|                     |\
#                / |		         | \
#      100Mb 1ms/  |		         |  \ 100Mb 1ms
#   s(3)-------'   |		         |   '---------d(3)
# 	           |		         |
#      100Mb 1ms   | 		         |    100Mb 1ms
#   s(4)-----------'		         '-------------d(4)


# Following sessions are created using create-session method:

#  Traffic  Source  Destination  Banwidth   Flowid     Route Followed
#     EF     s(0)     d(0)        5.0Mb       1      s(0)-e1-c1-e2-d(0)
#     AF1    s(1)     d(1)        3.5Mb       2      s(1)-e1-c1-e2-d(1)
#     AF1    s(2)     d(2)        4.5Mb       3      s(2)-e1-c2-e2-d(2)
#     AF1    s(3)     d(3)        3.0Mb       4      s(3)-e1-c2-e2-d(3)
#     BE     s(4)     d(4)        2.5Mb       5      unsuccessful as bandwith not available so forwarded with fid 0

# QOS Routing needs following two hooks from its environment
#
# 1. Link instproc get-bw{}
#    this should return the current available bandwith on the link 
#
# 2. RtModule/QOSPF/perd-bw-changed{}
#    the QOSR routing agents should be informed whenever the available 
#    bandwith on a link changes


ADC set bandwidth_ 0
ADC/Param set oresv_rate_ 0

Simulator instproc duplex-qosr-link { n1 n2 bw delay type} {
    $self duplex-link $n1 $n2 $bw $delay $type
    set link12 [$self link $n1 $n2]
    set link21 [$self link $n2 $n1]
    # admission control module
    $link12 set paramadc_ [new ADC/Param]
    $link21 set paramadc_ [new ADC/Param]
    [$link12 set paramadc_] set bandwidth_ $bw
    [$link21 set paramadc_] set bandwidth_ $bw
}

Simulator instproc simplex-qosr-link { n1 n2 bw delay type} {    
    $self simplex-link $n1 $n2 $bw $delay $type
    set link12 [$self link $n1 $n2]
    # admission control module
    $link12 set paramadc_ [new ADC/Param]
    [$link12 set paramadc_] set bandwidth_ $bw
}


#implement get-bw function required for qos routing
# available bw = total bandwith * utilization - reserved bandwith
SimpleLink instproc get-bw {} {
    $self instvar paramadc_
    if {![info exists paramadc_]} {
	return 0
    }
    set bandwidth_ [bw_parse [$paramadc_ set bandwidth_]]
      return [expr $bandwidth_ * \
  		[$paramadc_ set utilization_] - \
  		[$paramadc_ set oresv_rate_] ]
}

#shows adc parameters
SimpleLink instproc show-bw {} {
    $self instvar paramadc_
    if {![info exists paramadc_]} {
	return 0
    }
    set bandwidth_ [bw_parse [$paramadc_ set bandwidth_]]
    puts "$bandwidth_ : [$paramadc_ set utilization_] : [$paramadc_ set oresv_rate_]"
}

#not of much use------ just for debugging
Simulator instproc links {s d e1 c1 c2 e2} {
    upvar $s s1 $d d1
    for {set i 0} {$i < 5} {incr i} {
	set link [$self link $s1($i) $e1]
	$link show-bw
    }
    set link [$self link $e1 $c1]
    $link show-bw    
    set link [$self link $c1 $e2]
    $link show-bw       
    set link [$self link $e1 $c2]
    $link show-bw    
    set link [$self link $c2 $e2]
    $link show-bw       
    for {set i 0} {$i < 5} {incr i} {
	set link [$self link $e2 $d1($i)]
	$link show-bw
    }
}

# creates a session between $self and $dest of bandwith $bw with fid $fid
# bind agaent $agnt to that session.
# 1. get-perd-explicit-route with the traffic type $traffic
# 2. query admission control module along the path and make reservations
# 
Node instproc create-session {dest bw fid agnt traffic} {
    $self instvar src_agent_

    set rt [$self get-module "QOSPF"]
    set eroute [$rt get-perd-explicit-route $dest $bw $traffic]

    puts "eroute = $eroute"

    if {![llength $eroute]} {
	puts "create session $bw $fid [$self id] - [$dest id] unsuccessful: bandwidth not available"
	$agnt set class_ 0
	return 0
    }

    set sim [Simulator instance]
    set node1 $self
    set success 1 ;#whether reservation was successful
    set nlist "" ;#list of nodes where rollback has to be done in case

    foreach i $eroute {
	#get node from id
	set node2 [$sim get-node-by-id $i]
	#get link connecting node1 and node2
	set link [$sim link $node1 $node2]
	#get adc on them FIXME check whether it has adc
	set adc [$link set paramadc_]
	#reserve bandwith on link
	set ret [$adc admit_flow 0 $bw 0]
	#if ret false, rollback FIXME verify this
	if {!$ret} {
	    set success 0 ;#oops, we have to rollback
	    set rnode1 $self
	    foreach j $nlist {
		set rnode2 [$sim get-node-by-id $j]
		set rlink [$sim link $rnode1 $rnode2]
		set radc [$rlink set paramadc_]
		$radc rej_action 0 $bw 0
		set rnode1 $rnode2
	    }
	    break
	}
	lappend nlist $node2
	set node1 $node2
    } 
    
    if {!$success} {
	puts "create session $bw $fid [$self id] - [$dest id] unsuccessful: reservation unsuccessful"
	return 0 ;#sorry, failed to reserve
    }
    set node1 $self

    #so, reservation done
    #inform ls routing agent about bandwith changes
    $rt perd-bw-changed

    foreach i $eroute {
       #get node from id2]
       set node2 [$sim get-node-by-id $i]
       #get ls routing agent on the node
       set nrt2 [$node2 get-module "QOSPF"]
       #get link connecting node1 and node2
       set link [$sim link $node1 $node2]
       $nrt2 perd-bw-changed
       set node1 $node2
    } 

    #setup source routing
    $agnt set class_ $fid
    $agnt target $src_agent_
    eval "$src_agent_ install_connection $fid [$self id] [$dest id] [$self id] $eroute"
    
    puts "create session $bw $fid [$self id] - [$dest id] successful"
    return 1
}

# stops a session between $self and $dest of bandwith $bw
# agaent $agnt gets the route (get-path) along which the pakets are sent 
# along this path, the reservation is freed up & the application is stoppped.
# 
Node instproc stop-session {dest bw agnt appl} {
    $self instvar src_agent_

    set rt [$self get-module "QOSPF"]

    set myfid [$agnt set fid_] 
    if {$myfid  == 0} {
	puts "No reservation was done earlier from [$self id] - [$dest id] of $bw capacity"
	return 0
    }

    $agnt target $src_agent_
    set eroute [$src_agent_ get-path $myfid]

    puts "eroute = $eroute"
    if {![llength $eroute]} {
	puts "stop session $bw [$self id] - [$dest id] : unable to retrieve the route"
	return 0
    }

    $appl stop

    set sim [Simulator instance]

    for {set i 0} {$i < [llength $eroute]} {incr i} {
	if {[expr $i + 1] == [llength $eroute]} {
	    break
	}
	set k [lindex $eroute $i]
	set j [lindex $eroute [expr $i + 1]]
	set node1 [$sim get-node-by-id $k]
	set node2 [$sim get-node-by-id $j]
	#puts "$k $j $node1 $node2"
	set link [$sim link $node1 $node2]
	#$link show-bw
 	set radc [$link set paramadc_]
 	$radc rej_action 0 $bw 0
	#$link show-bw
    }

    #so, reservation is freed up
    #and inform ls routing agent about bandwith changes

    $rt perd-bw-changed

    foreach i $eroute {
        #get node from id2]
 	set node2 [$sim get-node-by-id $i]
 	#get ls routing agent on the node
 	set nrt2 [$node2 get-module "QOSPF"]
 	#get link connecting node1 and node2
 	set link [$sim link $node1 $node2]
 	$nrt2 perd-bw-changed
 	set node1 $node2
    } 

    puts "stop session $bw [$self id] - [$dest id] successful"
    return 1
}

set ns [new Simulator]

#enable source routing
$ns src_rting 1

$ns color 0 blue
$ns color 1 red
$ns color 2 green
$ns color 3 purple
$ns color 4 brown

set rate0 5000000
set rate1 6000000
set rate2 8000000
set rate3 7500000
set rate4 5000000

set sessionTime 1.0
set startTime 2.0
set stopsessionTime 84.0 
set testTime 85.0
set packetSize 1000

Node enable-module "QOSPF"

set s(0) [$ns node]
set s(1) [$ns node]
set s(2) [$ns node]
set s(3) [$ns node]
set s(4) [$ns node]

set e1 [$ns node]
set c1 [$ns node]
set c2 [$ns node]
set e2 [$ns node]

set d(0) [$ns node]
set d(1) [$ns node]
set d(2) [$ns node]
set d(3) [$ns node]
set d(4) [$ns node]

set f [open out.tr w]
$ns trace-all $f
set nf [open out.nam w]
$ns namtrace-all $nf

# create links
$ns duplex-qosr-link $s(0) $e1 100Mb 1ms DropTail
$ns duplex-qosr-link $s(1) $e1 100Mb 1ms DropTail
$ns duplex-qosr-link $s(2) $e1 100Mb 1ms DropTail
$ns duplex-qosr-link $s(3) $e1 100Mb 1ms DropTail
$ns duplex-qosr-link $s(4) $e1 100Mb 1ms DropTail

$ns simplex-qosr-link $e1 $c1 10Mb 5ms dsRED/edge
$ns simplex-qosr-link $c1 $e1 10Mb 5ms dsRED/core
$ns simplex-qosr-link $e2 $c1 10Mb 5ms dsRED/edge
$ns simplex-qosr-link $c1 $e2 10Mb 5ms dsRED/core
$ns simplex-qosr-link $e1 $c2 8Mb 4ms dsRED/edge
$ns simplex-qosr-link $c2 $e1 8Mb 4ms dsRED/core
$ns simplex-qosr-link $e2 $c2 8Mb 4ms dsRED/edge
$ns simplex-qosr-link $c2 $e2 8Mb 4ms dsRED/core

$ns duplex-qosr-link $e2 $d(0) 100Mb 10ms DropTail
$ns duplex-qosr-link $e2 $d(1) 100Mb 10ms DropTail
$ns duplex-qosr-link $e2 $d(2) 100Mb 10ms DropTail
$ns duplex-qosr-link $e2 $d(3) 100Mb 10ms DropTail
$ns duplex-qosr-link $e2 $d(4) 100Mb 10ms DropTail

set qE1C1 [[$ns link $e1 $c1] queue]
set qC1E1 [[$ns link $c1 $e1] queue]

$qE1C1 meanPktSize $packetSize
$qE1C1 set numQueues_ 3
$qE1C1 setNumPrec 3

$qE1C1 addPolicyEntry [$s(0) id] [$d(0) id] TokenBucket 46 2000000 10000
$qE1C1 addPolicyEntry [$s(1) id] [$d(1) id] trTCM 10 2000000 10000 3000000 10000
$qE1C1 addPolicyEntry [$s(2) id] [$d(2) id] trTCM 10 2000000 10000 3000000 10000
$qE1C1 addPolicyEntry [$s(3) id] [$d(3) id] trTCM 10 2000000 10000 3000000 10000
$qE1C1 addPolicyEntry [$s(4) id] [$d(4) id] TokenBucket 0 2000000 10000
$qE1C1 addPolicyEntry [$e1 id] -1 TokenBucket 46 2000000 10000
$qE1C1 addPolicyEntry [$c1 id] -1 TokenBucket 46 2000000 10000
$qE1C1 addPolicyEntry [$c2 id] -1 TokenBucket 46 2000000 10000
$qE1C1 addPolicyEntry [$e2 id] -1 TokenBucket 46 2000000 10000

$qE1C1 addPolicerEntry TokenBucket 46 47
$qE1C1 addPolicerEntry trTCM 10 12 14
$qE1C1 addPolicerEntry TokenBucket 0 50

$qE1C1 addPHBEntry 46 0 0
$qE1C1 addPHBEntry 47 0 1
$qE1C1 addPHBEntry 10 1 0
$qE1C1 addPHBEntry 12 1 1
$qE1C1 addPHBEntry 14 1 2   
$qE1C1 addPHBEntry 0 2 0
$qE1C1 addPHBEntry 50 2 1

$qE1C1 configQ 0 0 50 100 0.02
$qE1C1 configQ 0 1 0 0 1.00
$qE1C1 configQ 1 0 35 60 0.15
$qE1C1 configQ 1 1 15 25 0.30
$qE1C1 configQ 1 2 5 10 0.60
$qE1C1 configQ 2 0 10 20 0.25
$qE1C1 configQ 2 1 0 0 1.00

$qC1E1 meanPktSize $packetSize
$qC1E1 set numQueues_ 3
$qC1E1 setNumPrec 3

$qC1E1 addPHBEntry 46 0 0
$qC1E1 addPHBEntry 47 0 1
$qC1E1 addPHBEntry 10 1 0
$qC1E1 addPHBEntry 12 1 1
$qC1E1 addPHBEntry 14 1 2   
$qC1E1 addPHBEntry 0 2 0
$qC1E1 addPHBEntry 50 2 1

$qC1E1 configQ 0 0 50 100 0.02
$qC1E1 configQ 0 1 0 0 1.00
$qC1E1 configQ 1 0 35 60 0.15
$qC1E1 configQ 1 1 15 25 0.30
$qC1E1 configQ 1 2 5 10 0.60
$qC1E1 configQ 2 0 10 20 0.25
$qC1E1 configQ 2 1 0 0 1.00 

set qE2C1 [[$ns link $e2 $c1] queue]
set qC1E2 [[$ns link $c1 $e2] queue]

$qE2C1 meanPktSize $packetSize
$qE2C1 set numQueues_ 3
$qE2C1 setNumPrec 3

$qE2C1 addPolicyEntry [$d(0) id] [$s(0) id] TokenBucket 46 2000000 10000
$qE2C1 addPolicyEntry [$d(1) id] [$s(1) id] trTCM 10 2000000 10000 3000000 10000
$qE2C1 addPolicyEntry [$d(2) id] [$s(2) id] trTCM 10 2000000 10000 3000000 10000
$qE2C1 addPolicyEntry [$d(3) id] [$s(3) id] trTCM 10 2000000 10000 3000000 10000
$qE2C1 addPolicyEntry [$d(4) id] [$s(4) id] TokenBucket 0 2000000 10000
$qE2C1 addPolicyEntry [$e1 id] -1 TokenBucket 46 2000000 10000
$qE2C1 addPolicyEntry [$c1 id] -1 TokenBucket 46 2000000 10000
$qE2C1 addPolicyEntry [$c2 id] -1 TokenBucket 46 2000000 10000
$qE2C1 addPolicyEntry [$e2 id] -1 TokenBucket 46 2000000 10000

$qE2C1 addPolicerEntry TokenBucket 46 47
$qE2C1 addPolicerEntry trTCM 10 12 14
$qE2C1 addPolicerEntry TokenBucket 0 50

$qE2C1 addPHBEntry 46 0 0
$qE2C1 addPHBEntry 47 0 1
$qE2C1 addPHBEntry 10 1 0
$qE2C1 addPHBEntry 12 1 1
$qE2C1 addPHBEntry 14 1 2
$qE2C1 addPHBEntry 0 2 0
$qE2C1 addPHBEntry 50 2 1

$qE2C1 configQ 0 0 50 100 0.02
$qE2C1 configQ 0 1 0 0 1.00
$qE2C1 configQ 1 0 35 60 0.15
$qE2C1 configQ 1 1 15 25 0.30
$qE2C1 configQ 1 2 5 10 0.60
$qE2C1 configQ 2 0 10 20 0.25
$qE2C1 configQ 2 1 0 0 1.00

$qC1E2 meanPktSize $packetSize
$qC1E2 set numQueues_ 3
$qC1E2 setNumPrec 3

$qC1E2 addPHBEntry 46 0 0
$qC1E2 addPHBEntry 47 0 1
$qC1E2 addPHBEntry 10 1 0
$qC1E2 addPHBEntry 12 1 1
$qC1E2 addPHBEntry 14 1 2   
$qC1E2 addPHBEntry 0 2 0
$qC1E2 addPHBEntry 50 2 1

$qC1E2 configQ 0 0 50 100 0.02
$qC1E2 configQ 0 1 0 0 1.00
$qC1E2 configQ 1 0 35 60 0.15
$qC1E2 configQ 1 1 15 25 0.30
$qC1E2 configQ 1 2 5 10 0.60
$qC1E2 configQ 2 0 10 20 0.25
$qC1E2 configQ 2 1 0 0 1.00

set qE1C2 [[$ns link $e1 $c2] queue]
set qC2E1 [[$ns link $c2 $e1] queue]

$qE1C2 meanPktSize $packetSize
$qE1C2 set numQueues_ 3
$qE1C2 setNumPrec 3

$qE1C2 addPolicyEntry [$s(0) id] [$d(0) id] TokenBucket 46 2000000 10000
$qE1C2 addPolicyEntry [$s(1) id] [$d(1) id] trTCM 10 2000000 10000 3000000 10000
$qE1C2 addPolicyEntry [$s(2) id] [$d(2) id] trTCM 10 2000000 10000 3000000 10000
$qE1C2 addPolicyEntry [$s(3) id] [$d(3) id] trTCM 10 2000000 10000 3000000 10000
$qE1C2 addPolicyEntry [$s(4) id] [$d(4) id] TokenBucket 0 2000000 10000
$qE1C2 addPolicyEntry [$e1 id] -1 TokenBucket 46 2000000 10000
$qE1C2 addPolicyEntry [$c1 id] -1 TokenBucket 46 2000000 10000
$qE1C2 addPolicyEntry [$c2 id] -1 TokenBucket 46 2000000 10000
$qE1C2 addPolicyEntry [$e2 id] -1 TokenBucket 46 2000000 10000

$qE1C2 addPolicerEntry TokenBucket 46 47
$qE1C2 addPolicerEntry trTCM 10 12 14
$qE1C2 addPolicerEntry TokenBucket 0 50

$qE1C2 addPHBEntry 46 0 0
$qE1C2 addPHBEntry 47 0 1
$qE1C2 addPHBEntry 10 1 0
$qE1C2 addPHBEntry 12 1 1
$qE1C2 addPHBEntry 14 1 2   
$qE1C2 addPHBEntry 0 2 0
$qE1C2 addPHBEntry 50 2 1

$qE1C2 configQ 0 0 50 100 0.02
$qE1C2 configQ 0 1 0 0 1.00
$qE1C2 configQ 1 0 35 60 0.15
$qE1C2 configQ 1 1 15 25 0.30
$qE1C2 configQ 1 2 5 10 0.60
$qE1C2 configQ 2 0 10 20 0.25
$qE1C2 configQ 2 1 0 0 1.00

$qC2E1 meanPktSize $packetSize
$qC2E1 set numQueues_ 3
$qC2E1 setNumPrec 3

$qC2E1 addPHBEntry 46 0 0
$qC2E1 addPHBEntry 47 0 1
$qC2E1 addPHBEntry 10 1 0
$qC2E1 addPHBEntry 12 1 1
$qC2E1 addPHBEntry 14 1 2   
$qC2E1 addPHBEntry 0 2 0
$qC2E1 addPHBEntry 50 2 1

$qC2E1 configQ 0 0 50 100 0.02
$qC2E1 configQ 0 1 0 0 1.00
$qC2E1 configQ 1 0 35 60 0.15
$qC2E1 configQ 1 1 15 25 0.30
$qC2E1 configQ 1 2 5 10 0.60
$qC2E1 configQ 2 0 10 20 0.25
$qC2E1 configQ 2 1 0 0 1.00 

set qE2C2 [[$ns link $e2 $c2] queue]
set qC2E2 [[$ns link $c2 $e2] queue]

$qE2C2 meanPktSize $packetSize
$qE2C2 set numQueues_ 3
$qE2C2 setNumPrec 3

$qE2C2 addPolicyEntry [$d(0) id] [$s(0) id] TokenBucket 46 2000000 10000
$qE2C2 addPolicyEntry [$d(1) id] [$s(1) id] trTCM 10 2000000 10000 3000000 10000
$qE2C2 addPolicyEntry [$d(2) id] [$s(2) id] trTCM 10 2000000 10000 3000000 10000
$qE2C2 addPolicyEntry [$d(3) id] [$s(3) id] trTCM 10 2000000 10000 3000000 10000
$qE2C2 addPolicyEntry [$d(4) id] [$s(4) id] TokenBucket 0 2000000 10000
$qE2C2 addPolicyEntry [$e1 id] -1 TokenBucket 46 2000000 10000
$qE2C2 addPolicyEntry [$c1 id] -1 TokenBucket 46 2000000 10000
$qE2C2 addPolicyEntry [$c2 id] -1 TokenBucket 46 2000000 10000
$qE2C2 addPolicyEntry [$e2 id] -1 TokenBucket 46 2000000 10000

$qE2C2 addPolicerEntry TokenBucket 46 47
$qE2C2 addPolicerEntry trTCM 10 12 14
$qE2C2 addPolicerEntry TokenBucket 0 50

$qE2C2 addPHBEntry 46 0 0
$qE2C2 addPHBEntry 47 0 1
$qE2C2 addPHBEntry 10 1 0
$qE2C2 addPHBEntry 12 1 1
$qE2C2 addPHBEntry 14 1 2
$qE2C2 addPHBEntry 0 2 0
$qE2C2 addPHBEntry 50 2 1

$qE2C2 configQ 0 0 50 100 0.02
$qE2C2 configQ 0 1 0 0 1.00
$qE2C2 configQ 1 0 35 60 0.15
$qE2C2 configQ 1 1 15 25 0.30
$qE2C2 configQ 1 2 5 10 0.60
$qE2C2 configQ 2 0 10 20 0.25
$qE2C2 configQ 2 1 0 0 1.00 

$qC2E2 meanPktSize $packetSize
$qC2E2 set numQueues_ 3
$qC2E2 setNumPrec 3

$qC2E2 addPHBEntry 46 0 0
$qC2E2 addPHBEntry 47 0 1
$qC2E2 addPHBEntry 10 1 0
$qC2E2 addPHBEntry 12 1 1
$qC2E2 addPHBEntry 14 1 2   
$qC2E2 addPHBEntry 0 2 0
$qC2E2 addPHBEntry 50 2 1

$qC2E2 configQ 0 0 50 100 0.02
$qC2E2 configQ 0 1 0 0 1.00
$qC2E2 configQ 1 0 35 60 0.15
$qC2E2 configQ 1 1 15 25 0.30
$qC2E2 configQ 1 2 5 10 0.60
$qC2E2 configQ 2 0 10 20 0.25
$qC2E2 configQ 2 1 0 0 1.00

#configure qospf agent on all qospf nodes
$ns configure-perd-qospf-nodes

#create traffic sources

set udp0 [new Agent/UDP]
$ns attach-agent $s(0) $udp0
set cbr0 [new Application/Traffic/CBR]
$cbr0 attach-agent $udp0
$cbr0 set packet_size_ $packetSize
$udp0 set packetSize_ $packetSize
$cbr0 set rate_ $rate0
set null0 [new Agent/Null]
$ns attach-agent $d(0) $null0
$ns connect $udp0 $null0

set tcp1 [new Agent/TCP]
$ns attach-agent $s(1) $tcp1
set ftp1 [new Application/FTP]
$ftp1 attach-agent $tcp1
$ftp1 set packet_size_ $packetSize
$tcp1 set packetSize_ $packetSize
$ftp1 set rate_ $rate1
set null1 [new Agent/TCPSink]
$ns attach-agent $d(1) $null1
$ns connect $tcp1 $null1

set udp2 [new Agent/UDP]
$ns attach-agent $s(2) $udp2
set cbr2 [new Application/Traffic/CBR]
$cbr2 attach-agent $udp2
$cbr2 set packet_size_ $packetSize
$udp2 set packetSize_ $packetSize
$cbr2 set rate_ $rate2
set null2 [new Agent/Null]
$ns attach-agent $d(2) $null2
$ns connect $udp2 $null2

set udp3 [new Agent/UDP]
$ns attach-agent $s(3) $udp3
set cbr3 [new Application/Traffic/CBR]
$cbr3 attach-agent $udp3
$cbr3 set packet_size_ $packetSize
$udp3 set packetSize_ $packetSize
$cbr3 set rate_ $rate3
set null3 [new Agent/Null]
$ns attach-agent $d(3) $null3
$ns connect $udp3 $null3


set udp4 [new Agent/UDP]
$ns attach-agent $s(4) $udp4
set cbr4 [new Application/Traffic/CBR]
$cbr4 attach-agent $udp4
$cbr4 set packet_size_ $packetSize
$udp4 set packetSize_ $packetSize
$cbr4 set rate_ $rate4
set null4 [new Agent/Null]
$ns attach-agent $d(4) $null4
$ns connect $udp4 $null4

proc finish {} {
	global ns f nf
	$ns flush-trace
	close $f
	close $nf

	puts "running nam..."
	exec nam -r 20.6ms out.nam &
	exit 0
}

$qE1C1 printPolicyTable
$qE1C1 printPolicerTable

$ns at [expr $sessionTime + 0.0] "$s(0) create-session $d(0) 5.0Mb 1 $udp0 EF"
$ns at [expr $sessionTime + 0.2] "$s(1) create-session $d(1) 3.5Mb 2 $tcp1 AF1"
$ns at [expr $sessionTime + 0.4] "$s(2) create-session $d(2) 4.5Mb 3 $udp2 AF1"
$ns at [expr $sessionTime + 0.6] "$s(3) create-session $d(3) 3.0Mb 4 $udp3 AF1"
$ns at [expr $sessionTime + 0.8] "$s(4) create-session $d(4) 2.5Mb 5 $udp4 BE"

$ns at [expr $startTime + 0.0] "$cbr0 start"
$ns at [expr $startTime + 0.2] "$ftp1 start"
$ns at [expr $startTime + 0.4] "$cbr2 start"
$ns at [expr $startTime + 0.6] "$cbr3 start"
$ns at [expr $startTime + 0.8] "$cbr4 start"

$ns at 20.0 "$qE1C1 printStats"
$ns at 20.0 "$qC1E2 printStats"
$ns at 40.0 "$qE1C1 printStats"
$ns at 40.0 "$qC1E2 printStats"
$ns at 60.0 "$qE1C1 printStats"
$ns at 60.0 "$qC1E2 printStats"
$ns at 80.0 "$qE1C1 printStats"
$ns at 80.0 "$qC1E2 printStats"

#$ns at [expr $stopsessionTime - 0.2] "$ns links s d $e1 $c1 $c2 $e2"

$ns at [expr $stopsessionTime + 0.0] "$s(0) stop-session $d(0) 5.0Mb $udp0 $cbr0"
$ns at [expr $stopsessionTime + 0.2] "$s(1) stop-session $d(1) 3.5Mb $tcp1 $ftp1"
$ns at [expr $stopsessionTime + 0.4] "$s(2) stop-session $d(2) 4.5Mb $udp2 $cbr2"
$ns at [expr $stopsessionTime + 0.6] "$s(3) stop-session $d(3) 3.0Mb $udp3 $cbr3"
$ns at [expr $stopsessionTime + 0.8] "$s(4) stop-session $d(4) 2.5Mb $udp4 $cbr4"

#$ns at [expr $testTime + 0.5] "$ns links s d $e1 $c1 $c2 $e2"

$ns at [expr $testTime + 1.0] "finish"

$ns run

