
#send periodic LSA at interval: -1 for no periodic updates
Agent/QOSPF set advertInterval 300
#hold down time after an update has been sent: -1 for no hold down timer
Agent/QOSPF set advertHoldDownInterval 0
#send update when bw changes by advertMinBwChangePercent percent
Agent/QOSPF set advertMinBwChangePercent 10

#ondemand or precomputed routes
Agent/QOSPF set onDemand 1

#timer class to manage periodic updates
Class SendUpdateTimer -superclass Timer

SendUpdateTimer instproc init { ns agent } {
    $self next $ns
    $self instvar qospf_agent_
    $self set qospf_agent_ $agent
}

SendUpdateTimer instproc timeout {} {
    global rtglibRNG
    $self instvar qospf_agent_
    $qospf_agent_ instvar lastUpdateTime_ ns_
    Agent/QOSPF instvar advertInterval
    
    $qospf_agent_ send-periodic-update
    set lastUpdateTime_ [$ns_ now]
    if { $advertInterval > 0 } {
	$self sched [expr ([Agent/QOSPF set advertInterval] * \
			       [$rtglibRNG uniform 0.5 1.5])]
    }

}

Agent/QOSPF instproc init {{args "NONE"}} {
    $self instvar ns_ sendUpdateTimer_ peers_ lastUpdateTime_ mytype_

    $self next 

    $self set mytype_ $args
    #puts stderr "$mytype_"
    $self set ns_ [Simulator instance] 
    $self set sendUpdateTimer_ [new SendUpdateTimer [$self set ns_] $self]
    $self set peers_ ""
    $self set lastUpdateTime_ 0

    #puts "in Agent/rtProto/QOSPF instproc init: node = $node, self = $self"
    
    global rtglibRNG

    $self instvar LS_ready
    set LS_ready 0


    set updateTime [$rtglibRNG uniform 0.0 0.5]
    #$ns_ at $updateTime "$self send-periodic-update"
    $sendUpdateTimer_ sched $updateTime
}

Agent/QOSPF instproc send-periodic-update {} {
    $self instvar ns_ node_
    #puts "in Agent/QOSPF instproc send-periodic-update now = [$ns_ now] node = [$node_ id]"
    $self cmd sendUpdates
}


Agent/QOSPF instproc get-peers {} {
    $self instvar peers_ node_ mytype_
    
    #puts "in Agent/QOSPF instproc get-peers: node = $node_, peers_ = $peers_"

    #puts stderr "$mytype_"
    set peers ""
    foreach nbr $peers_ {
	set rtnbr [$nbr get-module QOSPF]
	if {$mytype_=="EF"} {
	set qagentnbr [$rtnbr set efqospf_agent_]
	} elseif {$mytype_=="AF1"} {
	set qagentnbr [$rtnbr set af1qospf_agent_]
	} elseif {$mytype_=="AF2"} {
	set qagentnbr [$rtnbr set af2qospf_agent_]
	} elseif {$mytype_=="AF3"} {
	set qagentnbr [$rtnbr set af3qospf_agent_]
	} elseif {$mytype_=="AF4"} {
	set qagentnbr [$rtnbr set af4qospf_agent_]
	} elseif {$mytype_=="BE"} {
	set qagentnbr [$rtnbr set beqospf_agent_]
	} else {
	set qagentnbr [$rtnbr set qospf_agent_]
	}
	lappend peers [$nbr id] 
	lappend peers [$qagentnbr set agent_addr_] 
	lappend peers [$qagentnbr set agent_port_]
    }

    set peers
}

Agent/QOSPF instproc get-node-id {} {
	$self instvar node_
	return [$node_ id]
}

Agent/QOSPF instproc get-links-status {} {
    $self instvar peers_ ns_ node_
    
    set linkstat ""
    
    foreach nbr [$node_ neighbors] {
	lappend linkstat [$nbr id]
	set link [$ns_ link $node_ $nbr]
 	if { [$link up?] == "up" } {
 	    lappend linkstat 1
 	} else {
 	    lappend linkstat 0
 	}
	lappend linkstat [$link cost?]
	lappend linkstat [$link get-bw]

    }
    set linkstat

}

Agent/QOSPF instproc get-delay-estimates {} {
    $self instvar node_ ns_
    set total_delays ""
    set packet_size 8000.0 ;# bits
    foreach nbr [$node_ neighbors] {
	set intf [$ns_ link $node_ $nbr]
	set q_limit [ [$intf queue ] set limit_]
	#FIXME check role of this bw
	set bw [bw_parse [ [$intf link ] set bandwidth_ ] ]
	    set p_delay [time_parse [ [$intf link ] set delay_] ]
	set total_delay [expr $q_limit * $packet_size / $bw + $p_delay]
	    $self cmd setDelay [$nbr id] $total_delay
    }
}


Agent/QOSPF instproc send-buffered-messages {} {
    $self instvar ns_ sendUpdateTimer_ lastUpdateTime_ node_
    $class instvar advertHoldDownInterval advertInterval

    set bufempty [$self cmd isBufferEmpty]

    if { $bufempty == 0 } {
	$self cmd send-buffered-messages
	set lastUpdateTime_ [$ns_ now]

    global rtglibRNG
  
	if { $advertInterval > 0 } {
	    $sendUpdateTimer_ resched [expr ([Agent/QOSPF set advertInterval] * \
						 [$rtglibRNG uniform 0.5 1.5]) ]
	}
    }
    
}

Agent/QOSPF instproc intf-changed {} {
    #puts "in Agent/QOSPF instproc intf-changed"
    $self instvar ns_ sendUpdateTimer_ lastUpdateTime_ node_
    $class instvar advertHoldDownInterval advertInterval

    $self cmd intfChanged

    #check if any buffered messages
    set bufempty [$self cmd isBufferEmpty]

    #puts "lastUpdateTime_ = $lastUpdateTime_"
    if { $bufempty == 0 } {
	#puts "[$node_ id]: there are updates to send"
	
	set time1 [expr $lastUpdateTime_ + $advertHoldDownInterval]
	set time2 [$ns_ now]

	if { $time1 > $time2 } {
	    #schedule for later
	    #puts "will send at $time1"
	    $ns_ at $time1 "$self send-buffered-messages"
	} else {
	    #puts "sending now $time2"
	    $self send-buffered-messages
	}
	
    } else {
	#puts "[$node_ id]: no updates to send"
    }
    
}

Agent/QOSPF instproc get-pre-explicit-route {dest bw} {
    set eroute [$self cmd get-pre-explicit-route [$dest id] [bw_parse $bw]]
    return $eroute
}

Agent/QOSPF instproc get-od-explicit-route {dest bw} {
    set eroute [$self cmd get-od-explicit-route [$dest id] [bw_parse $bw]]
    return $eroute
}

Agent/QOSPF instproc get-explicit-route {dest bw} {
    if { [Agent/QOSPF set onDemand] == 0 } {
	return [$self get-pre-explicit-route $dest $bw]
    } else {
	return [$self get-od-explicit-route $dest $bw]
    }
}

Agent/QOSPF instproc getAdvertMinBwChangePercent {} {
    $class instvar advertMinBwChangePercent
    return advertMinBwChangePercent
}

Agent/QOSPF instproc bw-changed {} {
    $self instvar lastUpdateTime_ node_

    $self intf-changed    
}
